package com.ruoyi.ss.transactionBill.service.impl;

import com.ruoyi.common.core.domain.BaseValidator;
import com.ruoyi.common.core.domain.ValidateResult;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.ss.channel.domain.SmChannel;
import com.ruoyi.ss.channel.service.ISmChannelService;
import com.ruoyi.ss.device.domain.vo.DeviceVO;
import com.ruoyi.ss.device.service.DeviceService;
import com.ruoyi.ss.model.domain.enums.ModelTag;
import com.ruoyi.ss.store.service.StoreValidator;
import com.ruoyi.ss.suit.domain.SuitVo;
import com.ruoyi.ss.suit.service.SuitService;
import com.ruoyi.ss.transactionBill.domain.TransactionBill;
import com.ruoyi.ss.transactionBill.service.TransactionBillService;
import com.ruoyi.ss.transactionBill.service.TransactionBillValidator;
import com.ruoyi.ss.user.service.UserValidator;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.Objects;

/**
 * 2024/4/22
 */
@Service
public class TransactionBillValidatorImpl extends BaseValidator implements TransactionBillValidator {

    @Autowired
    private DeviceService deviceService;

    @Autowired
    private UserValidator userValidator;

    @Autowired
    private ISmChannelService channelService;

    @Autowired
    private SuitService suitService;

    @Autowired
    private StoreValidator storeValidator;

    @Autowired
    private TransactionBillService transactionBillService;

    @Override
    public ValidateResult preAddOrder(TransactionBill data) {
        if (data == null) {
            return error("数据不能为空");
        }

        // 检查设备是否符合条件
        DeviceVO device = deviceService.selectByDeviceNo(data.getDeviceNo());
        if (device == null || device.getDeleted()) {
            return error("当前设备不存在");
        }
        if (StringUtils.isEmpty(device.getMac())) {
            return error("当前设备Mac为空，无法充值");
        }

        // 店铺
        if (!storeValidator.isExist(Collections.singletonList(device.getStoreId()))) {
            return error("当前设备店铺不存在，无法充值");
        }

        // 商户检查
        if (!userValidator.isUsage(device.getUserId())) {
            return error("当前设备商户不存在或不可用，无法充值，请确认商户账号是否正常");
        }

        // 套餐检查
        SuitVo suit = suitService.selectSuitBySuitId(data.getSuitId());
        if (suit == null) {
            return error("当前设备套餐不存在");
        }
        if (!Objects.equals(suit.getDeviceId(), device.getDeviceId())) {
            return error("当前套餐不属于该设备");
        }
        if (suit.getPrice() == null) {
            return error("当前套餐未配置价格，请联系商户处理");
        }
        if (suit.getPrice().compareTo(data.getMoney()) != 0) {
            return error("当前套餐价格已发生变化，请重新下单");
        }
        if (suit.getValue() == null || suit.getTimeUnit() == null) {
            return error("当前套餐未配置时间，请联系商户处理");
        }
        if (!suit.getValue().equals(data.getSuitTime())) {
            return error("当前套餐时间已发生变化，请重新下单");
        }
        if (!suit.getTimeUnit().equals(data.getSuitTimeUnit())) {
            return error("当前套餐时间已发生变化，请重新下单");
        }

        // 价格计算
        SmChannel channel = channelService.selectSmChannelByChannelId(data.getChannelId());
        if (channel == null || channel.getEnabled() == null || !channel.getEnabled()) {
            return error("不支持的支付方式");
        }

        return success();
    }

    /**
     * 获取订单详情前校验
     *
     * @param billId
     * @param userId
     * @return
     */
    @Override
    public ValidateResult preGetDetailByApp(Long billId, Long userId) {
        if (billId == null || userId == null) {
            return error("参数错误：billId与userId不允许为空");
        }

        if (!this.allowGet(billId, userId)) {
            return error("您不允许访问该订单");
        }

        return success();
    }

    /**
     * 允许查询订单
     * @param billId
     * @param userId
     * @return
     */
    private boolean allowGet(Long billId, Long userId) {
        TransactionBill bill = transactionBillService.selectSmTransactionBillByBillId(billId);
        if (bill != null ) {
            List<Long> allowUsers = Arrays.asList(bill.getUserId(), bill.getMchId());
            return allowUsers.contains(userId);
        }
        return false;
    }

    /**
     * 能否在线充值
     */
    @Override
    public boolean canRechargeOnline(Long deviceId) {
        DeviceVO device = deviceService.selectSmDeviceByDeviceId(deviceId);
        if (device == null) {
            return false;
        }
        // 4G和WIFI可以进行在线充值
        return device.getModelTags().contains(ModelTag.FOUR_G.getTag()) || device.getModelTags().contains(ModelTag.WIFI.getTag());
    }
}
